#include "geosim.h"
#include "gspriv.h"
#include "ietypes.h"
#include "plabw.h"
#include "windfunc.h"
#include "iemisc.h"
#include "editrec.h"
#include "ifw.h"
#include "windprop.h"
#include <string.h>
#ifndef MSDOS
#include <stdlib.h>
#endif

/* Internal protos */
static void plText(GSlistHandle l, int slot);
static char *plLabel(int slot);
static void plIn();
static void plClick(int slot);
static void ShowPlabStuff();
static void PSetupList();
static boolean TempLabIsValid();
static void ItemToTempFields(int);
static void TempFieldsToItem(int);
static void ClearPlabFields();
/**/

static EditRec *name, *func, *text;

static GSlistHandle labelList;
static GSsclistHandle labelScroll;
static int numPlabs, curPlab;

static BigString tempName, tempFunc, tempText;
static MenuItemStatusType tempActive;

static IEPopup *curPop;

static boolean TempLabIsValid()
{
	return (strlen(tempName)>0 && strlen(tempFunc)>0);
}

static void plText(GSlistHandle l, int slot)
{
	if (slot == curPlab)
		GSsetcolor(GSconvertcolor("red"));
	else
		GSsetcolor(GSconvertcolor("black"));
}

static char *plLabel(int slot)
{
	IEMenuItem *m;
	
	if (slot < numPlabs) {
		m = GetPlabelNum(slot, curPop);
		return m->name;
		}
		
	return "Error!";
}

static void plIn()
{
}

static void plClick(int slot)
{
	int old = curPlab;
	
	if (TempLabIsValid() && (curPlab != NO_ITEM))
		TempFieldsToItem(curPlab);
	
	curPlab = slot;
	
	GSdrawlistitem(labelList, New, old);
	GSdrawlistitem(labelList, New, curPlab);
	
	ItemToTempFields(curPlab);
	GSenablebutton("popupprop", "plabDelete", TRUE);
}

static void ItemToTempFields(int cur)
{
	IEMenuItem *m = GetPlabelNum(cur, curPop);
	if (!m)
		return;
		
	strcpy(tempName, m->name);
	strcpy(tempText, m->text);
	strcpy(tempFunc, m->funcname);
	tempActive = m->active;
	
	UpdateEditRec(name);
	UpdateEditRec(text);
	UpdateEditRec(func);
	GSsetbuttontext("popupprop", "plabToggleActive",
		(tempActive == ENABLED)? "Enabled" : "Disabled");
	GSenablebutton("popupprop", "plabToggleActive", TRUE);
}

static void TempFieldsToItem(int cur)
{
	IEMenuItem *m = GetPlabelNum(cur, curPop);
	if (!m)
		return;
	
	strcpy(m->name, tempName);
	strcpy(m->text, tempText);
	strcpy(m->funcname, tempFunc);
	m->active = tempActive;
	ClearPlabFields();
}

static void ClearPlabFields()
{
	tempName[0] = tempText[0] = tempFunc[0] = '\0';
	name->inspt = text->inspt = func->inspt = 0;
	tempActive = ENABLED;
	
	GSdisablebutton("popupprop", "plabDelete", TRUE);
	GSsetbuttontext("popupprop", "plabToggleActive",
		(tempActive == ENABLED)? "Enabled" : "Disabled");
	GSenablebutton("popupprop", "plabToggleActive", TRUE);
	UpdateEditRec(name);
	UpdateEditRec(text);
	UpdateEditRec(func);
}

void DrawPlabStuff(IEPopup *p)
{
	curPop = p;
	
	PSetupList();
				
	name = NewEditRec(tempName, BIGSTRINGLEN, 1, SMALL, 
		"popupprop", "plabName", c_identifier_filter, NULL);

	func = NewEditRec(tempFunc, BIGSTRINGLEN, 1, SMALL, 
		"popupprop", "plabFunc", c_identifier_filter, NULL);

	text = NewEditRec(tempText, BIGSTRINGLEN, 1, SMALL, 
		"popupprop", "plabText", NULL, NULL);
	
	ClearPlabFields();
	ShowPlabStuff();
}

void RedrawPlabStuff()
{
	ShowPlabStuff();
}

void RemovePlabStuff()
{
	if (TempLabIsValid()) {
		if (curPlab != NO_ITEM)
			TempFieldsToItem(curPlab);
		}
	SetPopupDims(curPop);
}

void DismantlePlabStuff()
{
	DisposEditRec(name);
	DisposEditRec(func);
	DisposEditRec(text);
	GSdismantlesclist(labelScroll);
}

void ShowPlabStuff()
{
	int x, y, wd, ht;
	
	UpdateEditRec(name);
	UpdateEditRec(text);
	UpdateEditRec(func);
	
	GSgetdragareaspecs("popupprop", "plabPick", 
				&x, &y, &wd, &ht, NULL, NULL);
	ShadowRect(x, y, wd, ht, THIN, Concave);
	GSdrawsclist(labelScroll);
}

void PSetupList()
{
	IEMenuItem *cur = curPop->itemlisthead;
	
	curPlab = NO_ITEM;
	numPlabs = 0;
	
	while (cur != NULL) {
		numPlabs++;
		cur = cur->nextmenuitem;
		}
	
	GSsclistsetup("popupprop", "plabPick",
					1,
					"plabpicklabel",
					0, 0,
					JUST_LEFT,
					plLabel,
					plText,
					plIn,
					plIn,
					plClick,
					"plabScroll",
					GSconvertcolor(THUMBCOL),
					GSconvertcolor(THUMBTL),
					GSconvertcolor(THUMBBR),
					numPlabs, 0,
					NULL,
					"plabUp",
					"plabDown",
					&labelList, &labelScroll);
}

boolean plab_scroll(int x, int y, DragStatusType s)
{
	GShandlesclist(labelScroll, x, y, s);
	return TRUE;
}

boolean plab_pick(int x, int y, DragStatusType s)
{
	GShandlelist(labelList, x, y, s);
	return TRUE;
}


boolean plab_btn(char *bn)
{
	int r = GET_INC(bn[4]);
	if (r > 0)
		GShandleupbutton(labelScroll);
	else
		GShandledownbutton(labelScroll);
}


boolean plab_name(int x, int y, DragStatusType s)
{
	if (/*s==MOUSE_INSIDE ||*/s==DRAG_FINAL || s==MOUSE_CLICK)
		SetCurrentEditRec(name);
}


boolean plab_text(int x, int y, DragStatusType s)
{
	if (/*s==MOUSE_INSIDE ||*/s==DRAG_FINAL || s==MOUSE_CLICK)
		SetCurrentEditRec(text);
}


boolean plab_func(int x, int y, DragStatusType s)
{
	if (/*s==MOUSE_INSIDE ||*/s==DRAG_FINAL || s==MOUSE_CLICK)
		SetCurrentEditRec(func);
}


boolean plab_toggle()
{
	tempActive = (tempActive == ENABLED)? DISABLED : ENABLED;
	
	GSsetbuttontext("popupprop", "plabToggleActive",
		(tempActive == ENABLED)? "Enabled" : "Disabled");
		
	GSenablebutton("popupprop", "plabToggleActive", TRUE);
}

boolean plab_new()
{
	int old = curPlab;
	
	if (TempLabIsValid()) {
		if (curPlab == NO_ITEM) {
		
			IEMenuItem *m = (IEMenuItem *) malloc(sizeof(IEMenuItem));
			
			if (m == NULL)
				GSelog(-1, "Out of memory for menu item.\n");
				
			if (curPop->itemlisthead == NULL)
				curPop->itemlisthead = m;
			else
				curPop->itemlisttail->nextmenuitem = m;
			curPop->itemlisttail = m;
			m->nextmenuitem = NULL;
			++numPlabs;
			TempFieldsToItem(numPlabs-1);
			GSsetsclistnumitems(labelScroll, numPlabs, TRUE);
			}
		else
			TempFieldsToItem(curPlab);
		}
		
	curPlab = NO_ITEM;
	ClearPlabFields();
	
	if (old != NO_ITEM)
		GSdrawlistitem(labelList, New, old);
}

boolean plab_delete()
{
	IEMenuItem *prev = NULL, *cur;
	
	if (curPlab < 0) {
		GSdisablebutton("popupprop", "plabDelete", TRUE);
		return;
		}
	
	if (curPlab > 0) {
		prev = GetPlabelNum(curPlab-1, curPop);
		cur = prev->nextmenuitem;
		prev->nextmenuitem = cur->nextmenuitem;
		}
	else {
		cur = curPop->itemlisthead;
		curPop->itemlisthead = cur->nextmenuitem;
		}
		
	if (curPop->itemlisttail == cur)
		curPop->itemlisttail = prev;
		
	free(cur);
	curPlab = NO_ITEM;
	GSsetsclistnumitems(labelScroll, --numPlabs, TRUE);
	ClearPlabFields();
}
